﻿using System;
using System.Windows;
using System.Windows.Controls.Primitives;
using System.Windows.Input;

namespace NullableComboBoxDemo
{
    /// <summary>
    /// This behavior allows giving a Selector control (ListBox, ComboBox, etc) a Nullify command.
    /// By iteself, adding NullableSelector.IsNullable=True will do nothing except register an 
    /// Attached property of type ICommand. The control's ControlTemplate must be modified to add a
    /// button making use of this NullifyCommand property. 
    /// </summary>
    /// <example>
    /// ComboBox usage:
    /// <ComboBox ItemsSource="{Binding Items}" NullableSelector.IsNullable="True"/>
    /// 
    /// ComboBox Control template:
    /// <Button x:Name="PART_NullifyButton" 
    ///         Command="{TemplateBinding NullableSelector.NullifyCommand}" 
    ///         Visibility="{TemplatBinding NullableSelector.IsNullable, Converter={StaticResource BooleanToVisibilityConverter}}"/>
    /// </example>
    public class NullableSelector
    {
        public static readonly DependencyProperty IsNullableProperty = DependencyProperty.RegisterAttached(
            "IsNullable", typeof (bool), typeof (NullableSelector), new PropertyMetadata(default(bool), OnIsNullableChanged));

        public static void SetIsNullable(DependencyObject element, bool value)
        {
            element.SetValue(IsNullableProperty, value);
        }

        public static bool GetIsNullable(DependencyObject element)
        {
            return (bool)element.GetValue(IsNullableProperty);
        }

        private static void OnIsNullableChanged(DependencyObject dependencyObject, DependencyPropertyChangedEventArgs e)
        {
            var selector = dependencyObject as Selector;
            if (selector == null)
                return;

            if ((bool)e.NewValue)
            {
                selector.SetValue(NullifyCommandProperty, new NullifyCommand(selector));
            }
            else
            {
                selector.ClearValue(NullifyCommandProperty);
            }
        }
        
        public static readonly DependencyProperty NullifyCommandProperty = DependencyProperty.RegisterAttached(
            "NullifyCommand", typeof (ICommand), typeof (NullableSelector), new PropertyMetadata(default(ICommand)));

        public static void SetNullifyCommand(DependencyObject element, ICommand value)
        {
            element.SetValue(NullifyCommandProperty, value);
        }

        public static ICommand GetNullifyCommand(DependencyObject element)
        {
            return (ICommand) element.GetValue(NullifyCommandProperty);
        }

        private class NullifyCommand : ICommand
        {
            private readonly Selector _selectorControl;

            public NullifyCommand(Selector selectorControl)
            {
                _selectorControl = selectorControl;
            }
            public void Execute(object parameter)
            {
                _selectorControl.SelectedItem = null;
            }

            public bool CanExecute(object parameter) { return true; }
            public event EventHandler CanExecuteChanged;
        }
    }
}
